use strict

var fs = require('fs');
var util = require('util');
var handlebars = require('handlebars');
var config = require('./schema2html.config');
var userConfig = {
    templates:{

    }
};

var cache  = {
    additionalSchemas:{},
    additionalData:{},
    templates:{}
};

/**
 * @class Schema2Html
 * @description Provides Utility to parse JSON schema files and output html or a handlebars template
 *
 *
 * @typedef Schema2HtmlOptions
 * @property {string} output Specify an output of html, handlebars or string
 *
 * 
 */

var Schema2Html = function(schema,data,options,callback){

    handlebars.registerHelper('ifEqual', function (v1, v2, options) {
           
            if (v1 === v2) {
              return options.fn(this);
            } else {
              return false;
            }
          });


    this.options = options || {}
    this.dataSrc = data || {};

    // set some defaults
    this.scope = null
    this.funcs = []; // run schema processing async, through a functions array
    this.currentStep = 0; // default current step
    this.html = ""; // setup empty string to append html
    this.dummy = ""; // setup empty string to append html
    this.scope = null;
    this.callback = callback;
    this.compile = false;//,compile;
    

    
    

    //this.schemaDirectory = schemaDirectory;
    this.additionalSchemas = {};
    this.options.id = schema.id || null;
    this.options.handlers = this.options.handlers || {};
    this.dataResolver = this.options.dataResolver || null;

    
    


}

Schema2Html.prototype.parse = function(callback){
    this.loadTemplates(); // load in templates
    this.userTemplates();
    this.addFormOpen(this.options);
    this.render(this.schema);
    this.execute();
}

/**
 * @memberOf Schema2Html
 * @method addFormOpen
 * @instance
 * @description Adds a form open tag
 */

Schema2Html.prototype.addFormOpen = function(options){
    
    var qFunc = function(callback){


        var template = cache.templates.startFormPut;
            
        var rendered = template(options);
        this.addHtml(rendered);
        callback(null);
    }

    this.funcs.push(qFunc);

    return qFunc;
}

/**
 * @memberOf Schema2Html
 * @method addFormClose
 * @instance
 * @description Adds a form close tag
 */

Schema2Html.prototype.addFormClose = function(options){
    var template = cache.templates.endForm;
        
    var rendered = template(options);
    this.addHtml(rendered)
}

/**
 * @memberOf Schema2Html
 * @method loadTemplates
 * @instance
 * @description
 */

Schema2Html.prototype.loadTemplates = function(){

    for(var name in config.templates){
        var template = config.templates[name];
        this.loadTemplate(name,__dirname + '/' +config.templates[name]);
    }

}

/**
 * @memberOf Schema2Html
 * @method userTemplates
 * @instance
 * @description
 */

Schema2Html.prototype.userTemplates = function(){
    


    for(var name in Schema2Html.userConfig.templates){
        var template = Schema2Html.userConfig.templates[name];
            
        

        this.loadTemplate(name,Schema2Html.userConfig.templates[name]);
    }
}

/**
 * @memberOf Schema2Html
 * @method loadTemplate
 * @instance
 * @description
 */

Schema2Html.prototype.loadTemplate = function(key,path){
    this.funcs.push(function(callback){

        fs.readFile(path,function(err,data){
            if(err){
                console.log(err);
            }
            if(data){
                
                cache.templates[key] = handlebars.compile(data.toString());
                callback(null);
            }
        })

    }.bind(this))
}

/*
* Simple Control Flow Implementation
*
*/

/**
 * @memberOf Schema2Html
 * @method execute
 * @instance
 * @description
 */

Schema2Html.prototype.execute = function(){
    this.funcs[this.currentStep].apply(this,[this.done.bind(this)]);

}

/*
* Control Flow Callback
*
*/

/**
 * @memberOf Schema2Html
 * @method done
 * @instance
 * @description
 */

Schema2Html.prototype.done = function(){
    this.currentStep++;

    if(this.currentStep == this.funcs.length){
        this.options.silent ?  console.log('++rendering in:' + (this.funcs.length) + ' steps++') : null;
        this.addFormClose(this.options);
        var compiled = this.html + '<div id="mock-content" style="display:none">' + this.dummy + '</div>';

        if(this.options.output == 'compiled'){

            var template = handlebars.compile(compiled);

            
            compiled = template({data:this.dataSrc,additionalSources:cache.additionalData});
        }else if(this.options.output == 'handlebars'){
            var template = handlebars.compile(compiled);
            compiled = template;
        }
        

        this.callback ? this.callback(null,compiled) : null;
    }else{
        this.execute();
    }
}


/*
* Main render loop
*
*/

/**
 * @memberOf Schema2Html
 * @method render
 * @description
 * @instance
 */

Schema2Html.prototype.render = function(schema,deep,scopeName,mock,ref){

    var nScope;
    var type = schema.type || null;
    var datasrc = schema.datasrc || null;
    var anyof = schema.anyOf || null;
    var id = schema.id     || null;
    var maxItems = schema.maxItems || null;
    var minItems = schema.minItems || null;
    var roles = schema.access || null;

    schema.options = schema.options || {};

    datasrc = schema.options.datasrc ? schema.options.datasrc : datasrc;

    deep = deep || 1;
    scopeName = scopeName || null;

    datasrc!=null ? this.addDataSource(scopeName,deep,datasrc,schema) : null; // add datasrc if present
    

    

    this.funcs.push(function(callback){
        this.log( new Array( deep ).join( '~' ) + scopeName);
        callback(null);
    }.bind(this));
    
    if(this.checkRole(roles) == true){  

        switch(type){
            case 'object':
                // this is a select list, and we need to check if there is external data needed
                this.renderObject(scopeName,deep,schema,mock,ref);
                break;
            case 'string':
                this.renderString(deep,scopeName,schema,mock,ref);
                break;
            case 'array':
                this.renderArray(deep,scopeName,schema,minItems,maxItems,mock,ref);
                break;
            default:
                schema.options.renderAs ? this.renderCustomHandler(deep,scopeName,schema,minItems,maxItems,mock,ref) : null;
                break;  
        }
    
    }
    
}

Schema2Html.prototype.renderCustomHandler = function(deep,scopeName,schema,minItems,maxItems,mock,ref){

    this.options.handlers[schema.options.renderAs] ? this.options.handlers[schema.options.renderAs].apply(this,[deep,scopeName,schema,minItems,maxItems,mock,ref]) : null;

}

Schema2Html.prototype.checkRole = function(roles){
    if(!roles){
        return true;
    }

    var _that = this;
    var pass = false;

    roles.forEach(function(r){
        console.log(r);
        if(_that.roles.indexOf(r) >-1 || r=='*') {
            pass = true;
        };
    });

    return pass;
}

/*
* Adds additional linked schema reference
*
*/

/**
 * @memberOf Schema2Html
 * @method addSchemaReference
 * @instance
 * @description
 */

Schema2Html.prototype.addSchemaReference = function(scopeName,deep,schemaReference,omitFromMatch,mock,ref,partOf){
        
    this.funcs.push(function(callback){


        this.log( new Array( deep + 1 ).join( '~' ) + 'adding {schemaSrc:'+schemaReference+'} for '+ scopeName);

        var stream = schemaReference.indexOf('.json') > -1 ? this.schemaDirectory + schemaReference : this.schemaDirectory + schemaReference + '.json';
        var key = omitFromMatch != null ? schemaReference.replace(omitFromMatch,'') : schemaReference;
        var _that = this;

        if(this.additionalSchemas[key]){
            this.additionalSchemas[key] = cache[key]
            callback();
        }else{
            fs.readFile(stream,function(err,data){
                
                if(err){
                    console.log(err);
                }

                if(data){
                    var remainingFunctions = _that.funcs.slice(_that.currentStep+1);
                    _that.funcs.splice(_that.currentStep+1,_that.funcs.length-_that.currentStep);

                    _that.additionalSchemas[key] = JSON.parse(data.toString());
                    

            
            

                    if(!partOf){ 
                        _that.startGroup(key,deep,{},true);
                        _that.startGroupItem(key+'[__i__]',deep,_that.additionalSchemas[key],true);
                        _that.render(_that.additionalSchemas[key],deep,scopeName + '[__i__]',true,_that.generateId(key));
                        _that.endGroupItem(key+'[__i__]',deep,true);
                        _that.endGroup(key,deep,true);
                    }else{
                        _that.startGroup(key,deep,{},false);
                        //_that.startGroupItem(key,deep,_that.additionalSchemas[key],false);
                        _that.render(_that.additionalSchemas[key],deep,scopeName ,false,_that.generateId(key));
                        //_that.endGroupItem(key,deep,false);
                        _that.endGroup(key,deep,false);

                        _that.startGroup(key,deep,{},true);
                        //_that.startGroupItem(key,deep,_that.additionalSchemas[key],true);
                        _that.render(_that.additionalSchemas[key],deep,scopeName ,true,_that.generateId(key));
                        //_that.endGroupItem(key,deep,true);
                        _that.endGroup(key,deep,true);
                    }

                    var funcsAt = _that.funcs.length;
                    

                    _that.funcs = _that.funcs.concat(remainingFunctions);

                    cache[key] = _that.additionalSchemas[key];
                    callback();
                }
            })
        }

    }.bind(this));
        //console.log(this.additionalSchemas[schemaReference]);
    

}

/*
*   Starts Group
*
*/

/**
 * @memberOf Schema2Html
 * @method startGroup
 * @instance
 * @description
 */

Schema2Html.prototype.startGroup = function(scopeName,deep,schema,mock,ref){
    this.funcs.push(function(callback){
        var newScope,
            groupId;
        
        groupId = this.generateId(scopeName);
        this.log('+{startGroup:' + groupId + '}+');

        schema.form = schema.form || {};

        var template = cache.templates.startGroup;
        
        var rendered = template({
            id:this.generateId(scopeName),
            name:scopeName,
            label:schema.title || null,
            title:schema.title || null,
            class:schema.form.class || null,
            ref:ref,
            mock:mock
        });

        if(schema.form.help){
            rendered = rendered + cache.templates.help({
                id:'help-' + this.generateId(scopeName),
                val:schema.form.help
            });
        }

        mock != true ? this.addHtml(rendered) : this.addMock(rendered);

        callback();
    }.bind(this));
    
}

/*
*   Ends Group
*
*/

/**
 * @memberOf Schema2Html
 * @method endGroup
 * @instance
 * @description
 */

Schema2Html.prototype.endGroup = function(scopeName,deep,mock,ref){
    this.funcs.push(function(callback){
        var newScope,
                groupId;
            
        groupId = this.generateId(scopeName);
        this.log('+{endGroup:' + groupId + '}+');
        var template = cache.templates.endGroup;
        
        var rendered = template({
            id:this.generateId(scopeName),
            name:scopeName
        });
        
        mock != true ? this.addHtml(rendered) : this.addMock(rendered);
        callback(null);
    }.bind(this));
}

/**
 * @memberOf Schema2Html
 * @method startGroupItem
 * @instance
 * @description
 */

Schema2Html.prototype.startGroupItem = function(scopeName,deep,schema,mock,ref){
    this.funcs.push(function(callback){
        var newScope,
            groupId;
        
        groupId = this.generateId(scopeName);
        this.log('+{startGroup:' + groupId + '}+');

        schema.form = schema.options || {};
        schema.form = schema.form || {};


        var template = cache.templates.startGroupItem;
        
        var rendered = template({
            id:this.generateId(scopeName),
            name:scopeName,
            label:schema.title || null,
            title:schema.title || null,
            class:schema.form.class || null,
            deep:deep,
            ref:ref,
            mock:mock
        });

        if(schema.form.help){
            rendered = rendered + cache.templates.help({
                id:'help-' + this.generateId(scopeName),
                val:schema.form.help
            });
        }

        mock != true ? this.addHtml(rendered) : this.addMock(rendered);

        callback();
    }.bind(this));
    
}

/*
*   Ends Group
*
*/

/**
 * @memberOf Schema2Html
 * @method endGroupItem
 * @instance
 * @description
 */

Schema2Html.prototype.endGroupItem = function(scopeName,deep,mock,ref){
    this.funcs.push(function(callback){
        var newScope,
                groupId;
            
        groupId = this.generateId(scopeName);
        this.log('+{endGroup:' + groupId + '}+');
        var template = cache.templates.endGroupItem;
        
        var rendered = template({
            id:this.generateId(scopeName),
            name:scopeName,
            ref:ref
        });
        
        mock != true ? this.addHtml(rendered) : this.addMock(rendered);
        callback(null);
    }.bind(this));
}


/*
*   Starts Group Array
*
*/

/**
 * @memberOf Schema2Html
 * @method startGroupArray
 * @instance
 * @description
 */

Schema2Html.prototype.startGroupArray = function(scopeName,deep,schema,renderAmount,minItems,maxItems,mock,ref){
    this.funcs.push(function(callback){
        var newScope,
                groupId;
            
        groupId = this.generateId(scopeName);
        schema.form = schema.form || {};
        this.log('+{startGroupArray:' + groupId + '}+');
        var template = cache.templates.startGroupArray;
        
        var rendered = template({
            id:this.generateId(scopeName),
            name:scopeName,
            title:schema.title || "",
            total:renderAmount,
            minItems:minItems,
            maxItems:maxItems,
            class:schema.form.class || null,
            ref:ref,
            mock:mock,
            deathwatch:schema.form.deathwatch
        });
        mock != true ? this.addHtml(rendered) : this.addMock(rendered);

        callback(null);
    }.bind(this));
}

Schema2Html.prototype.renderMockItem = function(scopeName,deep,schema,mock){

}

/*
*   Ends Group Array
*
*/

/**
 * @memberOf Schema2Html
 * @method endGroupArray
 * @instance
 * @description
 */

Schema2Html.prototype.endGroupArray = function(scopeName,deep,mock,schema,ref){
    this.funcs.push(function(callback){
        var newScope,
                groupId;
        schema.form = schema.form || {};
        groupId = this.generateId(scopeName);
        this.log('+{endGroupArray:' + groupId + '}+');
        var template = cache.templates.endGroupArray;
        
        var rendered = template({
            id:this.generateId(scopeName),
            name:scopeName,
            addlabel:schema.form.addlabel || "add",
            ref:ref,
            mock:mock,
            customButton:schema.form.customButton
        });

        mock != true ? this.addHtml(rendered) : this.addMock(rendered);
        callback(null);
    }.bind(this));
}

/*
*   Adds DataSource
*   
*/

/**
 * @memberOf Schema2Html
 * @method addDataSource
 * @instance
 * @description
 */

Schema2Html.prototype.addDataSource = function(scopeName,deep,datasrc,schema){
    
    this.funcs.push(function(callback){
        this.log( new Array( deep + 1 ).join( '~' ) + 'adding {dataSrc:'+datasrc+'} for '+ scopeName);
        
        

        

        var type = datasrc.split('*');
        var url = type[1];
        var datahash = this.hash(datasrc);

        if(type[0] != 'json'){ 

            
            this.dataResolver[type[0]].retrieve(url,'GET',{},function(err,code,data){
                cache.additionalData[datahash] = data;
                callback(null);
            });
        
        }else{
            cache.additionalData[datahash] = schema.dataJson;

            
            callback(null);
        }

        

    }.bind(this));
    

}

/*
*   Renders JSON Schema object type
*
*/

/**
 * @memberOf Schema2Html
 * @method renderObject
 * @instance
 * @description
 */

Schema2Html.prototype.renderObject = function(scopeName,deep,schemaObject,mock,ref){

    
        var newScope,
            groupId;
        
        this.startGroup(scopeName,deep,schemaObject,mock,ref);
            
            deep++;
        // loop through object properties
        for(var name in schemaObject.properties){
            newScope = scopeName != null ? scopeName + '['+name+']' : name;
            // send each object back to the renderer
            this.render(schemaObject.properties[name],deep,newScope,mock,ref);



            if(schemaObject.properties[name].anyOf){ 
                console.log('caught any of');
                var schemaLength = schemaObject.properties[name].anyOf.length;
                var matchOn = schemaObject.properties[name].match || null;
                var omitFromMatch = schemaObject.properties[name].omit || null;
                
                for(i=0; i<schemaLength; i++){
                    var ref = schemaObject.properties[name].anyOf[i];
                    if(ref['$ref']){
                        
                        if(!matchOn){ 
                            this.addSchemaReference(name,deep,ref['$ref'],omitFromMatch,mock,ref,true);
                        }else if( this.dataValue(matchOn,false) == ref['$ref'].replace(omitFromMatch,"") ){
                            // this is a single match
                            this.addSchemaReference(name,deep,ref['$ref'],omitFromMatch,mock,ref,true);

                        }
                    }
                }
            }
        }


        this.endGroup(scopeName,deep,mock,ref);

    

}

/**
 * @memberOf Schema2Html
 * @method addHtml
 * @instance
 * @description
 */

Schema2Html.prototype.addHtml = function(html){
    this.html = this.html + html;
}

/**
 * @memberOf Schema2Html
 * @method addMock
 * @instance
 * @description
 */

Schema2Html.prototype.addMock = function(html){
    this.dummy = this.dummy + html;
}

/*
* Renders JSON Schema string type
*
*/

/**
 * @memberOf Schema2Html
 * @method renderString
 * @instance
 * @description
 */

Schema2Html.prototype.renderString = function(deep,scopeName,schema,mock,ref){
    
    this.funcs.push(function(callback){
        
        schema.form = schema.form != null ? schema.form : {};
        schema.form.class = schema.form.class || null;
        schema.form.format = schema.datasrc != null ? 'selectList' : schema.form.format;

        var elementId = this.generateId(scopeName);
        var val = this.dataValue(scopeName,this.compile);
        console.log(val);
        var template = schema.form.format ? cache.templates[schema.form.format] :cache.templates.textfield;

        if(schema.form.format && !mock){
            if(config.display[schema.form.format] && config.display[schema.form.format].nodata ){
                if(!this.dataValue(scopeName)){
                    template = cache.templates[config.display[schema.form.format].nodata];
                }
                if(!mock){ 

                    this.renderString(deep,scopeName + '[nodata]',schema,true);
                }
            }
        }else if(mock){
            if(config.display[schema.form.format] && config.display[schema.form.format].nodata ){
                //scopeName = scopeName.replace(/[0-9]/g, '__i__');
                template = cache.templates[config.display[schema.form.format].nodata]; //  has no data, present no data template
            }
        }
        
        // handle select list
        if(schema.datasrc){ 
            var datahash = this.hash(schema.datasrc);
            var src = "{{#additionalSources."+datahash+"}}<option value='{{key}}' {{#ifEqual ../"+ val.split('{{').join('').split('}}').join('') +" key}}selected='true'{{/ifEqual}}>{{value}}</option>{{/additionalSources."+datahash+"}}";
            
        }else{
            var src = null;
        }

        var rendered = "";

        if(schema.form.help){
            rendered = rendered + cache.templates.help({
                id:'help-' + this.generateId(scopeName),
                val:schema.form.help
            });
        }

        var rendered = rendered + template({
            id:this.generateId(scopeName),
            name:scopeName,
            label:schema.title || null,
            title:schema.title || null,
            val:val,
            src:src,
            required:schema.required || false,
            pattern:schema.form.pattern || null,
            errorMsg:schema.form.errorMsg || null,
            message:schema.form.message || null,
            class:schema.form.class,
            default:schema.form.default,
            override:schema.form.override,
            options:schema.form
        });

        mock != true ? this.addHtml(rendered) : this.addMock(rendered);


        this.log( new Array( deep + 1 ).join( '~' ) + scopeName + ' as {string} with {id:'+elementId+'} {val:' + val +'}');
        callback(null);
    }.bind(this));

}

/**
 * @memberOf Schema2Html
 * @method renderSelectList
 * @instance
 * @description
 */

Schema2Html.prototype.renderSelectList = function(deep,scopeName,schema,mock,ref){

}

/*
*   Renders SubSchema/Linked Schema and call the render loop again, merges in new functions to current control flow
*
*/

/**
 * @memberOf Schema2Html
 * @method renderSubSchema
 * @instance
 * @description
 */

Schema2Html.prototype.renderSubSchema = function(scopeName,deep,i,schema,mock,ref){
    this.funcs.push(function(callback){
        
        var schemaLength = schema.items.anyOf.length;
        var matchOn = schema.items.match || null;
        var omitFromMatch = schema.items.omit || null;

        var funcsAt = this.funcs.length;
        var remainingFunctions = this.funcs.slice(this.currentStep+1);
        this.funcs.splice(this.currentStep,this.funcs.length-this.currentStep);     
        
        newScope = scopeName != null ? scopeName + '['+i+']' : scopeName;
                
        //newScope = scopeName != null ? scopeName + '['+i+']' : scopeName;
                
        if(matchOn){
            var matchValue = this.dataValue(scopeName + '['+i+'][' + matchOn +']');
                matchValue = omitFromMatch != null ? matchValue.replace(omitFromMatch,'') : matchValue;

                if(this.additionalSchemas[matchValue]){
                    newScope = scopeName != null ? scopeName + '['+i+']' : scopeName;
                    this.log(new Array( deep + 1 ).join( '~' ) + 'Rendering Sub Schema for:' + newScope );
                    
                    var schemaMatch = this.additionalSchemas[matchValue];
                    //this.renderSubSchema(schemaMatch,deep,newScope); // async version
                    //this.startGroupItem(newScope,deep,schemaMatch,mock);
                    this.render(schemaMatch,deep,newScope,mock,this.generateId(matchValue));
                    //this.endGroupItem(newScope,deep,mock);

                }

        }


        
        
        this.funcs = this.funcs.concat(remainingFunctions);
        this.log('functions increased by:' + (this.funcs.length - funcsAt));

        callback();
    }.bind(this));
}

/*
*   Pass through function for console log, used to it appears in control flow
*
*/

/**
 * @memberOf Schema2Html
 * @method log
 * @instance
 * @description
 */

Schema2Html.prototype.log = function(message){
    if(!Schema2Html.silent){
        this.funcs.push(function(callback){
            console.log(message);
            callback();
        }.bind(this));
    }
}

/*
*   Renders JSON Schema array type
*
*/

/**
 * @memberOf Schema2Html
 * @method renderArray
 * @instance
 * @description
 */

Schema2Html.prototype.renderArray = function(deep,scopeName,schema,minItems,maxItems,mock,ref){
    
    
    

    this.log( new Array( deep + 1 ).join( '~' ) + scopeName + ' as {array} with {minItems:' + minItems + '} {maxItems:' + maxItems + '}' );
    
    var newScope,
        groupId,
        val,
        i;

    val = this.dataValue(scopeName);
    groupId = this.generateId(scopeName);

    renderAmt = val != null ? val.length : minItems; 

    renderAmt > minItems ? this.log(new Array( deep + 1 ).join( '~' ) + 'Modifiying {minItems} to {minItems:'+renderAmt+'} ') : null;

    renderAmt = renderAmt == 0 ? 1 : renderAmt;
    this.startGroupArray(scopeName,deep,schema,renderAmt,minItems,maxItems,mock,ref);
    


    if(renderAmt > 0){
        if(schema.items.anyOf){
            // need to fetch our schema possibilities
            var schemaLength = schema.items.anyOf.length;
            var matchOn = schema.items.match || null;
            var omitFromMatch = schema.items.omit || null;
            for(i=0; i<schemaLength; i++){
                var ref = schema.items.anyOf[i];
                if(ref['$ref']){
                    this.addSchemaReference(scopeName,deep,ref['$ref'],omitFromMatch,mock,ref);
                }
            }


            for(i=0; i<renderAmt; i++){
                            newScope = scopeName != null ? scopeName + '['+i+']' : scopeName;
                            this.startGroupItem(newScope,deep,schema,mock,ref);
                            this.renderSubSchema(scopeName,deep,i,schema,mock,ref); // async version
                            this.endGroupItem(newScope,deep,mock,ref);
                            //this.render(schemaMatch,deep,newScope);

            }


        }else{ 
            for(i=0; i<renderAmt; i++){
                newScope = scopeName != null ? scopeName + '['+i+']' : scopeName;
                newScope = mock == true ? scopeName + '[__i__]' : newScope;
                this.startGroupItem(newScope,deep,schema.items,mock,ref);
                this.render(schema.items,deep,newScope,mock,ref);
                this.endGroupItem(newScope,deep,mock,ref);
            }

            newScope = scopeName != null ? scopeName + '[__i__]' : scopeName;
            
            // render out dummy item
            //this.startGroupItem(scopeName + '[item]',deep,schema.items,true);
            schema.items.title = null;
            this.startGroup(newScope + '[group]',deep,schema.items,true,ref);
            this.startGroupItem(newScope,deep,schema.items,true,ref);
            this.render(schema.items,deep,newScope,true,ref);
            this.endGroupItem(newScope,deep,true,ref);
            this.endGroup(newScope,deep,true,ref);
            //this.endGroup(newScope,deep,true);
        }
    }else{
        if(schema.items.anyOf){

            var schemaLength = schema.items.anyOf.length;
            var matchOn = schema.items.match || null;
            var omitFromMatch = schema.items.omit || null;
            for(i=0; i<schemaLength; i++){
                var ref = schema.items.anyOf[i];
                if(ref['$ref']){
                    this.addSchemaReference(scopeName,deep,ref['$ref'],omitFromMatch,mock);
                }
            }
        }
    }

    this.endGroupArray(scopeName,deep,mock,schema,ref);



}

/**
 * @memberOf Schema2Html
 * @method generateId
 * @instance
 * @description
 */

Schema2Html.prototype.generateId = function(scopeName){
    return scopeName != null ? scopeName.toLowerCase().split('[').join('-').split(']').join('').split('.').join('-') : null;
}

/**
 * @memberOf Schema2Html
 * @method hash
 * @instance
 * @description
 */

Schema2Html.prototype.hash = function(hash){
    return hash.toLowerCase().replace(/ /g,'-').replace(/[^\w-]+/g,'');
}

/**
 * @memberOf Schema2Html
 * @method dataValue
 * @instance
 * @description
 */

Schema2Html.prototype.dataValue = function(scopeName,t){
    return this.byString(this.dataSrc,scopeName,true);
}

/**
 * @memberOf Schema2Html
 * @method byString
 * @instance
 * @description
 */

Schema2Html.prototype.byString = function(o, s, t) {
   if(o && typeof o != 'string'){ 
    s = s.replace(/\[(\w+)\]/g, '.$1'); // convert indexes to properties
    s = s.replace(/^\./, '');           // strip a leading dot
    var a = s.split('.');
        if(!t){ 
            while (a.length) {
                var n = a.shift();
                if (n in o) {
                    o = o[n];
                } else {
                    return;
                }
            }
        }
        return t == true ? '{{'+"data."+s+'}}' :o;
    }else{
        return null;
    }   
}

Schema2Html.userConfig = userConfig;
Schema2Html.silent = true;



module.exports = Schema2Html;